#include <ctype.h>
#include <string.h>
#include <errno.h>
#include <dos.h>
#include <sys/stat.h>

#include <gvision.h>

#define min(X, Y) ((X) < (Y) ? (X) : (Y))

#define _MAX_DRIVE MAXDRIVE
#define _MAX_DIR   MAXDIR
#define _MAX_FNAME MAXFILE
#define _MAX_EXT   MAXEXT
#define _MAX_PATH  MAXPATH

int fnsplit(const char *path, char *drive, char *dir, char *name, char *ext)
{
  int flags = 0, i, j;

  if (path[1] == ':')
   {
    if (drive)
     {
      drive[0] = path[0];
      drive[1] = ':';
      drive[2] = '\0';
     }
    flags |= DRIVE;
    path += 2;
   }
  else if (drive)
   *drive = '\0';

  for (i = j = 0; path[j]; j++)
   if (path[j] == '/' || path[j] == '\\')
    i = j + 1;

  if (i)
   {
    if (dir)
     {
      memcpy(dir, path, j=min(MAXDIR-1, i));
      dir[j] = '\0';
     }
    flags |= DIRECTORY;
    path += i;
   }
  else if (dir)
   *dir = '\0';

  for (i = 0; path[i]; i++)
   if (path[i] == '.')
    break;

  if (i)
   {
    if (name)
     {
      memcpy(name, path, j=min(MAXFILE-1, i));
      name[j] = '\0';
     }
    flags |= FILENAME;
    path += i;
   }
  else if (name)
   *name = '\0';

  if (*path)
   {
    if (ext)
     {
      strncpy(ext, path, MAXEXT-1);
      ext[MAXEXT-1] = '\0';
     }
    flags |= EXTENSION;
   }
  else if (ext)
   *ext = '\0';

  return flags;
}

void fnmerge (char *path, const char *drive, const char *dir,
                          const char *name, const char *ext)
{
  if (drive && *drive)
   {
    *path++ = *drive;
    *path++ = ':';
    *path = '\0';
   }

  if (dir && *dir)
   {
    strncpy(path, dir, MAXDIR);
    while (*path) path++;
    if (*(path-1) != '\\')
     {
      *path++ = '\\';
      *path = '\0';
     }
   }

  if (name && *name)
   {
    strncpy(path, name, MAXFILE);
    while (*path) path++;
   }

  if (ext && *ext)
   {
     if (*ext != '.')
      *path++ = '.';
    strncpy(path, ext, MAXEXT);
   }
}

int getcurdir(int drive, char *buffer)
{
  char tmp[MAXPATHLEN+1], *p;
  int oldrive;

  if (drive)
   {
    oldrive = getdrive();
    chdrive(drive-1);
   }

  getcwd(tmp, MAXPATHLEN);
  
  p = tmp;
  do{
    p++;
    *buffer++ = ((*p == '/') ? '\\' : *p);
   }while(*p);

  if (drive)
   chdrive(oldrive);

  return 0;
}

void chdrive(int d)
{
 union REGS r;

 r.h.ah = 0x0e;
 r.h.dl = d;
 intdos(&r, &r);
}

int getdrive(void)
{
 union REGS r;

 r.h.ah = 0x19;
 intdos(&r, &r);
 return r.h.al;
}

int strnicmp(const char *str1, const char *str2, int count)
{
 register char *s = (char *)str1, *t = (char *)str2;
 register int i = 0;

 while (i<count)
  {
   if (!(*s)) return 0;

   if (tolower(*s) != tolower(*t))
    break;

   i++; s++; t++;
  }

 return (tolower(*s) - tolower(*t));

}

long filelength(int fd)
{
 struct stat buf;
 
 fstat(fd, &buf);
 return buf.st_size;
}